<?php

namespace Drupal\lb_auto_translation\HookHandlers;

use Drupal\block_content\BlockContentInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\lb_auto_translation\Traits\TranslationHtmlTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form handler (alter) for the Node form.
 *
 * @see lb_translation_form_node_form_alter().
 */
class NodeFormHandler implements ContainerInjectionInterface {

  use TranslationHtmlTrait;

  /**
   * Field names that will be translated.
   */
  const FIELDS = [
    'field_cta_button_label',
    'field_subtitle',
    'field_title',
    'field_spotlight_title',
    'field_sub_title',
    'field_hero_banner_title',
    'field_product_tab',
    'field_vertical_body',
  ];

  /**
   * Field types that will be translated.
   */
  const FIELD_TYPES = ['text', 'text_long', 'text_with_summary'];

  use StringTranslationTrait;

  /**
   * Block storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $blockStorage;

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = new static();
    $instance->blockStorage = $container->get('entity_type.manager')->getStorage('block_content');

    return $instance;
  }

  /**
   * Form alter for the Node form.
   *
   * @param array $form
   *   Drupal form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Drupal form state.
   * @param string $form_id
   *   Form id.
   */
  public function alter(array &$form, FormStateInterface $form_state, string $form_id):void {
    /** @var \Drupal\node\NodeInterface $node */
    $node = $form_state->getFormObject()->getEntity();

    if ($node->isTranslatable() && $node->isNewTranslation()) {
      $form['actions']['submit']['#submit'][] = [$this, 'submitTranslation'];
    }
  }

  /**
   * Form submit handler.
   *
   * @param array $form
   *   Drupal form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Drupal form state.
   */
  public function submitTranslation(array $form, FormStateInterface $form_state):void {
    /** @var \Drupal\node\NodeInterface $node */
    $node = $form_state->getFormObject()->getEntity();
    $langcode = $node->language()->getId();
    $original_langcode = $node->getUntranslated()->language()->getId();

    if (!$node->hasField('layout_builder__layout') || $node->get('layout_builder__layout')->isEmpty()) {
      return;
    }

    $sections = $node->get('layout_builder__layout')->getSections();

    foreach ($sections as $section) {
      foreach ($section->getComponents() as $component) {
        $config = $component->get('configuration');

        if (str_starts_with($config['id'], 'inline_block')) {
          $block_revision_id = $config['block_revision_id'];

          /** @var BlockContentInterface $translated_block */
          $translated_block = $this->blockStorage->loadRevision($block_revision_id);
          $this->translateBlockFields($translated_block, $original_langcode, $langcode);
        }
      }
    }
  }

  /**
   * Translated block fields.
   *
   * @param BlockContentInterface $block
   *   Block entity.
   * @param string $from
   *   Language from.
   * @param string $to
   *   Language to.
   */
  public function translateBlockFields(BlockContentInterface $block, string $from, string $to):void {
    foreach ($block->getFields() as $field_name => $field) {
      $type = $field->getFieldDefinition()->getType();

      if (in_array($type, self::FIELD_TYPES) ||
        in_array($field_name, self::FIELDS)) {
        $this->translateField($field, $from, $to);
        $block->save();
      }
    }
  }

  /**
   * Translates entity field.
   *
   * @param FieldItemListInterface $field
   *   Text string.
   * @param string $from
   *   Langcode from.
   * @param string $to
   *   Langcode to.
   */
  public function translateField(FieldItemListInterface $field, string $from, string $to):void {
    if ($field->isEmpty()) {
      return;
    }

    $type = $field->getFieldDefinition()->getType();

    switch ($type) {
      case 'link':
        $link_value = $field->getValue();

        foreach ($link_value as $key => $value) {
          $translated_title = $this->translate($value['title'], $from, $to);

          // Update the link title with the translated value.
          $link_value[$key]['title'] = $translated_title;
          $field->setValue($link_value);
        }

        break;

      case 'entity_reference_revisions':
        $reference_type = $field->getFieldDefinition()->getSettings()['target_type'];
        $storage = \Drupal::service('entity_type.manager')->getStorage($reference_type);

        $field_value = $field->getValue();

        foreach ($field_value as $key => $value) {
          $reference_entity = $storage->loadRevision($value['target_revision_id']);

          foreach ($reference_entity->getFields() as $field_name => $field) {
            $type = $field->getFieldDefinition()->getType();

            if (in_array($type, self::FIELD_TYPES) ||
              in_array($field_name, self::FIELDS)) {
              $this->translateField($field, $from, $to);
              $reference_entity->save();
            }
          }
        }

        break;

      default:
        $field_value = $field->getValue();

        foreach ($field_value as $key => $value) {
          $translated_value = $this->translate($value['value'], $from, $to);
          $field_value[$key]['value'] = $translated_value;

          if (!empty($value['format'])) {
            $field_value[$key]['format'] = $value['format'];
          }

          $field->setValue($field_value);
        }
    }
  }

  /**
   * Translates text string.
   *
   * @param string $text
   *   Text string.
   * @param string $from
   *   Langcode from.
   * @param string $to
   *   Langcode to.
   *
   * @return string
   *   Translated string.
   */
  public function translate(string $text, string $from, string $to):string {
    [$text, $tags] = $this->protectHtmlTags($text);
    $translated_value = \Drupal::service('auto_translation.utility')
      ->translate($text, $from, $to);

    return $this->restoreHtmlTags($translated_value, $tags);
  }

}
