<?php

namespace Drupal\Tests\lb_auto_translation\Unit;

use Drupal\auto_translation\Utility;
use Drupal\block_content\BlockContentInterface;
use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\Core\Entity\ContentEntityFormInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\layout_builder\Field\LayoutSectionItemList;
use Drupal\layout_builder\Section;
use Drupal\layout_builder\SectionComponent;
use Drupal\lb_auto_translation\HookHandlers\NodeFormHandler;
use Drupal\node\NodeInterface;
use Drupal\paragraphs\ParagraphInterface;
use Drupal\Tests\UnitTestCase;

/**
 * Unit test for NodeFormHandler class.
 *
 * @coversDefaultClass \Drupal\lb_translation\HookHandlers\NodeFormHandler
 * @group lb_translation
 */
class NodeFormHandlerTest extends UnitTestCase {

  /**
   * The mocked block storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $blockStorage;

  /**
   * The mocked block storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $paragraphStorage;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $paragraph = $this->createMock(ParagraphInterface::class);
    $paragraph->method('getFields')
      ->willReturn([]);

    // Mock the entity storage.
    $this->blockStorage = $this->createMock(EntityStorageInterface::class);
    $this->paragraphStorage = $this->createMock(EntityStorageInterface::class);

    $this->paragraphStorage->method('loadRevision')
      ->willReturn($paragraph);

    $field_definition1 = $this->createMock(FieldDefinitionInterface::class);
    $field_definition1->method('getType')->willReturn('text');
    $field1 = $this->createMock(FieldItemListInterface::class);
    $field1->method('getFieldDefinition')->willReturn($field_definition1);
    $field1->method('getValue')->willReturn([0 => ['value' => 'House', 'format' => 'full_html']]);
    $field_definition2 = $this->createMock(FieldDefinitionInterface::class);
    $field_definition2->method('getType')->willReturn('custom_text');
    $field2 = $this->createMock(FieldItemListInterface::class);
    $field2->method('getFieldDefinition')->willReturn($field_definition2);
    $field2->method('getValue')->willReturn([0 => ['value' => 'House', 'format' => 'full_html']]);

    $translated_block = $this->createMock(BlockContentInterface::class);
    $translated_block->method('getFields')
      ->willReturn([
        'field_custom_title' => $field1,
        'field_title' => $field2,
      ]);
    $this->blockStorage->method('loadRevision')
      ->willReturn($translated_block);

    // Mock the container services.
    $container = new ContainerBuilder();
    $entityTypeManager = $this->createMock(EntityTypeManagerInterface::class);
    $entityTypeManager->method('getStorage')
      ->willReturnMap([
        ['block_content', $this->blockStorage],
        ['paragraph', $this->paragraphStorage],
      ]);

    $autoTranslation = $this->createMock(Utility::class);
    $autoTranslation->method('translate')
      ->willReturnMap([
        ['CTA Button', 'en', 'nl', 'CTA Knop'],
        ['[#0][#1]House[#2][#3]', 'en', 'nl', '[#0][#1]Huis[#2][#3]'],
        ['House', 'en', 'nl', 'Huis'],
      ]);

    $container->set('entity_type.manager', $entityTypeManager);
    $container->set('auto_translation.utility', $autoTranslation);
    \Drupal::setContainer($container);
  }

  /**
   * Tests the ::create() method.
   */
  public function testCreate(): void {
    $instance = NodeFormHandler::create(\Drupal::getContainer());
    $this->assertInstanceOf(NodeFormHandler::class, $instance);
  }

  /**
   * Tests the alter() method when the node is translatable.
   *
   * @covers ::alter
   */
  public function testAlter(): void {
    // Mock a form array.
    $form = ['actions' => ['submit' => ['#submit' => []]]];

    // Mock the form state.
    $formState = $this->createMock(FormStateInterface::class);

    // Mock the node.
    $node = $this->createMock(\Drupal\node\NodeInterface::class);
    $node->method('isTranslatable')->willReturn(true);
    $node->method('isNewTranslation')->willReturn(true);

    // Mock the form object.
    $formObject = $this->createMock(ContentEntityFormInterface::class);
    $formObject->method('getEntity')->willReturn($node);

    $formState->method('getFormObject')->willReturn($formObject);

    $form_id = 'node_form';

    $handler = NodeFormHandler::create(\Drupal::getContainer());
    $handler->alter($form, $formState, $form_id);

    $this->assertContains([$handler, 'submitTranslation'], $form['actions']['submit']['#submit']);
  }

  /**
   * @covers ::submitTranslation
   */
  public function testSubmitTranslation() {
    $form_state = $this->createMock(FormStateInterface::class);

    $form_object = $this->createMock(ContentEntityFormInterface::class);

    $node = $this->createMock(NodeInterface::class);

    $form_state->expects($this->once())
      ->method('getFormObject')
      ->willReturn($form_object);
    $form_object->expects($this->once())
      ->method('getEntity')
      ->willReturn($node);

    $node->method('language')
      ->willReturnOnConsecutiveCalls(
        $this->getLanguageMock('nl'),
        $this->getLanguageMock('en'),
      );
    $node->expects($this->once())
      ->method('getUntranslated')
      ->willReturn($node);
    $node->expects($this->once())
      ->method('hasField')
      ->with('layout_builder__layout')
      ->willReturn(true);

    $field = $this->createMock(LayoutSectionItemList::class);
    $node->expects($this->exactly(2))
      ->method('get')
      ->with('layout_builder__layout')
      ->willReturn($field);
    $field->expects($this->once())
      ->method('isEmpty')
      ->willReturn(false);

    $section = $this->createMock(Section::class);
    $component = $this->createMock(SectionComponent::class);
    $config = ['id' => 'inline_block:test', 'block_revision_id' => 123];
    $component->expects($this->once())
      ->method('get')
      ->with('configuration')
      ->willReturn($config);
    $section->expects($this->once())
      ->method('getComponents')
      ->willReturn([$component]);
    $field->expects($this->once())
      ->method('getSections')
      ->willReturn([$section]);

    $handler = NodeFormHandler::create(\Drupal::getContainer());
    $handler->submitTranslation([], $form_state);
  }

  /**
   * @covers ::submitTranslation
   */
  public function testSubmitTranslationEmptyLayout() {
    $form_state = $this->createMock(FormStateInterface::class);
    $form_object = $this->createMock(ContentEntityFormInterface::class);
    $node = $this->createMock(NodeInterface::class);

    $form_state->expects($this->once())
      ->method('getFormObject')
      ->willReturn($form_object);
    $form_object->expects($this->once())
      ->method('getEntity')
      ->willReturn($node);

    $node->method('language')
      ->willReturnOnConsecutiveCalls(
        $this->getLanguageMock('nl'),
        $this->getLanguageMock('en'),
      );
    $node->expects($this->once())
      ->method('getUntranslated')
      ->willReturn($node);
    $node->expects($this->once())
      ->method('hasField')
      ->with('layout_builder__layout')
      ->willReturn(FALSE);

    $handler = NodeFormHandler::create(\Drupal::getContainer());
    $handler->submitTranslation([], $form_state);
  }

  private function getLanguageMock(string $langcode) {
    $language = $this->createMock(\Drupal\Core\Language\LanguageInterface::class);
    $language->method('getId')
      ->willReturn($langcode);
    return $language;
  }

  /**
   * @covers ::translateBlockFields
   */
  function testTranslateBlockFields() {
    $field_definition1 = $this->createMock(FieldDefinitionInterface::class);
    $field_definition1->method('getType')->willReturn('text');
    $field1 = $this->createMock(FieldItemListInterface::class);
    $field1->method('getFieldDefinition')->willReturn($field_definition1);
    $field1->method('getValue')->willReturn([0 => ['value' => 'House', 'format' => 'full_html']]);
    $field_definition2 = $this->createMock(FieldDefinitionInterface::class);
    $field_definition2->method('getType')->willReturn('custom_text');
    $field2 = $this->createMock(FieldItemListInterface::class);
    $field2->method('getFieldDefinition')->willReturn($field_definition2);
    $field2->method('getValue')->willReturn([0 => ['value' => 'House', 'format' => 'full_html']]);

    $block = $this->createMock(BlockContentInterface::class);
    $block->expects($this->exactly(1))->method('getFields')
      ->willReturn([
        'field_custom_title' => $field1,
        'field_title' => $field2,
      ]);

    $handler = NodeFormHandler::create(\Drupal::getContainer());
    $handler->translateBlockFields($block, 'en', 'nl');
  }

  /**
   * @covers ::translateField
   * @dataProvider translateFieldDataProvider
   */
  function testTranslateField(bool $empty, string $field_type, string $field_name, array $field_value, array $field_settings, array $nested_fields, string $from, string $to) {
    $handler = NodeFormHandler::create(\Drupal::getContainer());
    $field = $this->createMock(FieldItemListInterface::class);

    if ($empty) {
      $field->method('isEmpty')->willReturn(TRUE);
      $field->expects($this->never())
        ->method('getFieldDefinition');
      $handler->translateField($field, $from, $to);
    }
    elseif ($field_type === 'entity_reference_revisions') {
      $field->method('isEmpty')->willReturn(FALSE);
      $paragraph_fields = [];

      foreach ($nested_fields as $nested_field) {
        $paragraph_field = $this->createMock(FieldItemListInterface::class);

        if ($nested_field[0]) {
          $paragraph_field->method('isEmpty')->willReturn(TRUE);
          $paragraph_field->expects($this->never())
            ->method('getFieldDefinition');
        }
        else {
          $paragraph_field_definition = $this->createMock(FieldDefinitionInterface::class);
          $paragraph_field_definition->method('getType')->willReturn($nested_field[1]);
          $paragraph_field_definition->method('getSettings')->willReturn($nested_field[4]);
          $paragraph_field->method('isEmpty')->willReturn(FALSE);
          $paragraph_field->method('getFieldDefinition')
            ->willReturn($paragraph_field_definition);
          $paragraph_field->method('getValue')->willReturn($nested_field[3]);
        }

        $paragraph_fields[$nested_field[2]] = $paragraph_field;
      }

      $paragraph = $this->createMock(ParagraphInterface::class);
      $paragraph->method('getFields')
        ->willReturn($paragraph_fields);

      // Mock the entity storage.
      $blockStorage = $this->createMock(EntityStorageInterface::class);
      $paragraphStorage = $this->createMock(EntityStorageInterface::class);

      $paragraphStorage->method('loadRevision')
        ->willReturn($paragraph);

      // Mock the container services.
      $container = new ContainerBuilder();
      $entityTypeManager = $this->createMock(EntityTypeManagerInterface::class);
      $entityTypeManager->method('getStorage')
        ->willReturnMap([
          ['block_content', $blockStorage],
          ['paragraph', $paragraphStorage],
        ]);

      $autoTranslation = $this->createMock(Utility::class);
      $autoTranslation->method('translate')
        ->willReturnMap([
          ['CTA Button', 'en', 'nl', 'CTA Knop'],
          ['[#0][#1]House[#2][#3]', 'en', 'nl', '[#0][#1]Huis[#2][#3]'],
        ]);

      $container->set('entity_type.manager', $entityTypeManager);
      $container->set('auto_translation.utility', $autoTranslation);
      \Drupal::setContainer($container);

      $field_definition = $this->createMock(FieldDefinitionInterface::class);
      $field_definition->method('getType')->willReturn($field_type);
      $field_definition->method('getSettings')->willReturn($field_settings);
      $field->method('isEmpty')->willReturn(FALSE);
      $field->method('getFieldDefinition')
        ->willReturn($field_definition);
      $field->method('getValue')->willReturn($field_value);

      $handler = NodeFormHandler::create(\Drupal::getContainer());
      $handler->translateField($field, $from, $to);
    }
    else {
      $field_definition = $this->createMock(FieldDefinitionInterface::class);
      $field_definition->method('getType')->willReturn($field_type);
      $field_definition->method('getSettings')->willReturn($field_settings);
      $field->method('isEmpty')->willReturn(FALSE);
      $field->method('getFieldDefinition')
        ->willReturn($field_definition);
      $field->method('getValue')->willReturn($field_value);
      $handler->translateField($field, $from, $to);
    }
  }

  function translateFieldDataProvider() {
    return [
      [TRUE, 'text', 'field_title', [], [], [], 'en', 'nl'],
      [FALSE, 'text', 'field_title', [0 => ['value' => '<a href="/home-page"><span>House</span></a>', 'format' => 'full_html']], [], [], 'en', 'nl'],
      [FALSE, 'link', 'field_cta', [0 => ['title' => 'CTA Button']], [], [], 'en', 'nl'],
      [FALSE, 'entity_reference_revisions', 'field_tabs', [0 => ['target_revision_id' => '456']], ['target_type' => 'paragraph'], [
        [TRUE, 'text', 'field_title', [], [], [], 'en', 'nl'],
        [FALSE, 'text', 'field_title', [0 => ['value' => '<a href="/home-page"><span>House</span></a>', 'format' => 'full_html']], [], [], 'en', 'nl'],
        [FALSE, 'link', 'field_cta', [0 => ['title' => 'CTA Button']], [], [], 'en', 'nl'],
      ], 'en', 'nl'],
    ];
  }

  /**
   * @covers ::translate
   */
  function testTranslate() {
    $text = '<a href="/home"><span>House</span></a>';
    $translated_text = '<a href="/home"><span>Huis</span></a>';
    $from = 'en';
    $to = 'nl';
    $handler = NodeFormHandler::create(\Drupal::getContainer());
    $this->assertEquals($translated_text, $handler->translate($text, $from, $to));
  }

}
